<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use KangaPay\Payment;

if (!defined('_PS_VERSION_')) exit;

include_once _PS_MODULE_DIR_ . 'kangapay/classes/kangaPayLoader.php';

class KangaPay extends PaymentModule {
  const TABLE_NAME = 'order_kangapay';
  const CONFIG_KEYS = ['KANGA_PAY_API', 'KANGA_PAY_SECRET', 'KANGA_PAY_DEVEL' ];

  public $apiKey;
  public $secret;
  public $devel;
  public $showFee;

  private $moduleConfigurations = [];
  private $shop;

  protected $_html = '';

  public function __construct() {
    $this->name = 'kangapay';
    $this->tab = 'payments_gateways';
    $this->version = '0.1.0';
    $this->author = 'Paweł Stolarski';
    $this->need_instance = 1;
    $this->controllers = array( 'checknow', 'notification', 'payment', 'validation' );
    $this->is_eu_compatible = 1;
    $this->ps_versions_compliancy = [
      'min' => '1.7.0.0',
      'max' => _PS_VERSION_,
    ];

    $this->bootstrap = true;
    $this->currencies = true;
    $this->currencies_mode = 'checkbox';

    parent::__construct();

    $this->displayName = $this->l( 'Pay by Kanga Pay' );
    $this->description = $this->l( 'Turn on the ability to pay by Kanga Pay service.' );
    $this->confirmUninstall = $this->l( 'Are you sure you want to uninstall?' );


    /* Additional data */


    $this->moduleConfigurations = [
      'API' => new \KangaPay\Configuration\ApiConfiguration(),
    ];

    $this->shop = KangaPay\ShopConfiguration::fromConfiguration( Configuration::getMultiple([
      'KANGA_PAY_API',
      'KANGA_PAY_SECRET',
      'KANGA_PAY_DEVEL',
      'KANGA_PAY_PREV_PAYMENTS'
    ]) );

    $config = Configuration::getMultiple( self::CONFIG_KEYS );

    try {
      KangaPay\Integration::setShopInformation( $this->shop );
    } catch (Exception $ex) {}

    if (isset( $config[ 'KANGA_PAY_API' ] ))       $this->apiKey   = $config[ 'KANGA_PAY_SECRET' ];
    if (isset( $config[ 'KANGA_PAY_SECRET' ] ))    $this->secret   = $config[ 'KANGA_PAY_SECRET' ];
    if (isset( $config[ 'KANGA_PAY_DEVEL' ] ))     $this->devel    = $config[ 'KANGA_PAY_DEVEL' ];

    if (!isset( $this->apiKey ) || !isset( $this->secret ) || empty( $this->apiKey ) || empty( $this->secret )) {
      $this->warning = $this->trans('The "Payee" and "Address" fields must be configured before using this module.', array(), 'Modules.Checkpayment.Admin');
    }

    if (!count(Currency::checkPaymentCurrencies($this->id))) {
      $this->warning = $this->l( 'No currency has been set for this module.' );
    }
  }

  public function install() {
    try {
      // return parent::install();
      return parent::install()
        && $this->registerHook( 'displayHeader' )
        && $this->registerHook( 'paymentOptions' )
        && $this->registerHook( 'orderConfirmation' )
        && $this->createKangaPayOrderTable()
        && \KangaPay\OrderStatus::installKangaPayStatuses();
    } catch (Exception $ex) {
      return false;
    }
  }

  public function uninstall() {
    return $this->moduleConfigurations[ 'API' ]->uninstall()
      && $this->dropKangaPayOrderTable()
      && parent::uninstall();
  }

  private function processFormConfiguration( $configurationKey ) {
    if (!Tools::isSubmit( $configurationKey . 'btn' )) return;

    $postErrors = $this->moduleConfigurations[ $configurationKey ]->validation( $this );

    if (count( $postErrors )) {
      foreach ($postErrors as $err) $this->_html .= $this->displayError( $err );
      return;
    }

    $this->moduleConfigurations[ $configurationKey ]->saveConfiguration();
    $this->_html .= $this->displayConfirmation( $this->l( 'Settings updated' ) );
  }

  public function getContent() {
    $this->_html = '';

    $this->processFormConfiguration( 'API' );

    // $this->_html .= $this->_displayCheck();
    $this->_html .= $this->moduleConfigurations[ 'API' ]->renderForm( $this );

    return $this->_html;
  }



  /*\
   *   Payment module methods below
  \*/



  public function hookDisplayHeader( $params ) {
    // var_dump('hello Pay hookDisplayHeader');

    $this->context->controller->addCSS( 'modules/' . $this->name . '/views/css/kangapay.css' );
    $this->context->controller->addJS( 'modules/' . $this->name . '/views/js/kangapay.js' );
  }

  public function hookPaymentOptions( $params ) {
    if (!$this->active) return;
    if (!$this->checkCurrency( $params[ 'cart' ] ) ) return;

    $paymentOption = new PaymentOption();

    return [
      $paymentOption->setCallToActionText( $this->l( 'Pay by Kanga Pay' ) )
        ->setForm( $this->generatePaymentForm() )
        ->setAdditionalInformation( $this->context->smarty->fetch( 'module:kangapay/views/templates/payment_intro.tpl' ) )
    ];
  }

  public function hookOrderConfirmation( $params ){
    if (!$this->active || Tools::getValue( 'module' ) != 'kangapay') return;

    $paymentCheck = \KangaPay\Payment::confirmPayment(
      \KangaPay\ShopConfiguration::fromDefaultConfiguration(),
      $params['order']
    );

    if ($this->shop->isPrevPayments()) {
      \KangaPay\Payment::checkPreviousOrders( $this->shop );
    }

    $this->smarty->assign([
      'state'       => $paymentCheck[ 'state' ],
      'status'      => $paymentCheck[ 'state' ] == 'ERROR' ? 'fail' : 'ok',
      'contact_url' => $this->context->link->getPageLink( 'contact', true ),
      'orderRef'    => $params[ 'order' ]->reference,
      'checknowUrl' => $this->shop->getCheckUrl( $params[ 'order' ]->reference ),
    ]);

    return $this->fetch( 'module:kangapay/views/templates/payment_return.tpl' );
  }

  public function checkCurrency( $cart ) {
    $currency_order = new Currency( $cart->id_currency );
    $currencies_module = $this->getCurrency( $cart->id_currency );

    if (is_array( $currencies_module )) {
      foreach ($currencies_module as $currency_module) {
        if ($currency_order->id == $currency_module[ 'id_currency' ]) return true;
      }
    }

    return false;
  }

  protected function generatePaymentForm() {
    $ctx = $this->context;

    $ctx->smarty->assign([
      'action'  => $ctx->link->getModuleLink( $this->name, 'validation', [], true ),
      'err'     => Tools::getValue( 'err' ),
    ]);

    return $ctx->smarty->fetch( 'module:kangapay/views/templates/form.tpl' );
  }

  protected function createKangaPayOrderTable() {
    return Db::getInstance()->execute( 'CREATE TABLE IF NOT EXISTS `' . pSQL(_DB_PREFIX_) . self::TABLE_NAME . '` ('
      . '  `id_order` INT(10) UNSIGNED PRIMARY KEY'
      . ', `paymentId` VARCHAR(48) NOT NULL'
      . ', `state` VARCHAR(10) NULL'
    . ')' );
  }

  private function dropKangaPayOrderTable() {
    return Db::getInstance()->execute( 'DROP TABLE IF EXISTS `' . pSQL(_DB_PREFIX_) . self::TABLE_NAME . '`' );
  }
}
